const { pascalize } = require('humps');
const createTypes = require('../sourceNodes/createTypes');
const { prefixId, CODES } = require('../../errorMap');

const { getLoader } = require('../../utils');

module.exports = async (
  {
    actions,
    getNode,
    getNodesByType,
    webhookBody,
    reporter,
    parentSpan,
    schema,
    store,
    cache,
  },
  {
    apiToken,
    previewMode,
    environment,
    apiUrl,
    instancePrefix,
    localeFallbacks: rawLocaleFallbacks,
    localesToGenerate,
    pageSize,
    logApiCalls,
  },
) => {
  const localeFallbacks = rawLocaleFallbacks || {};

  if (!apiToken) {
    const errorText = `API token must be provided!`;
    reporter.panic(
      {
        id: prefixId(CODES.MissingAPIToken),
        context: { sourceMessage: errorText },
      },
      new Error(errorText),
    );
  }

  const loader = await getLoader({
    cache,
    apiToken,
    previewMode,
    environment,
    apiUrl,
    pageSize,
    logApiCalls,
    loadStateFromCache: !!process.env.GATSBY_WORKER_ID,
  });

  const context = {
    entitiesRepo: loader.entitiesRepo,
    actions,
    getNode,
    getNodesByType,
    localeFallbacks,
    localesToGenerate,
    schema,
    store,
    cache,
    generateType: type => {
      return `DatoCms${instancePrefix ? pascalize(instancePrefix) : ''}${type}`;
    },
  };

  console.log('--------- CREATE SCHEMA CUSTOMIZATION --------');
  console.log(
    JSON.stringify(
      { webhookBody, workerId: process.env.GATSBY_WORKER_ID },
      null,
      2,
    ),
  );
  console.log('--------- CREATE SCHEMA CUSTOMIZATION --------');

  const isWebhook = webhookBody && Object.keys(webhookBody).length > 0;

  if (!process.env.GATSBY_WORKER_ID && !isWebhook) {
    let activity;

    activity = reporter.activityTimer(`loading DatoCMS schema`, {
      parentSpan,
    });

    activity.start();

    await loader.load();
    await loader.saveStateToCache(cache);

    activity.end();
  }

  createTypes(context);
};
